require "rss/rss"

module RSS

	class Converter
		
		include Utils

		def initialize(to_enc, from_enc=nil)
			to_enc = to_enc.downcase.gsub('-', '_')
			from_enc ||= 'utf-8'
			from_enc = from_enc.downcase.gsub('-', '_')
			if to_enc == from_enc
				def_same_enc()
			else
				if respond_to?("def_from_#{from_enc}_to_#{to_enc}")
					send("def_from_#{from_enc}_to_#{to_enc}")
				else
					def_else_enc(from_enc, to_enc)
				end
			end
		end

		def convert(value)
			value
		end

		def def_convert()
			instance_eval(<<-EOC, *get_file_and_line_from_caller(0))
			def convert(value)
				if value.kind_of?(String)
					#{yield('value')}
				else
					value
				end
			end
			EOC
		end
		
		def def_else_enc(from_enc, to_enc)
			raise UnknownConvertMethod.new(from_enc, to_enc)
		end
		
		def def_same_enc()
			def_convert do |value|
				value
			end
		end

		def def_from_utf_8_to_euc_jp
      begin
        require "uconv"
      rescue LoadError
        raise UnknownConvertMethod.new('utf_8', 'euc_jp')
      end
			def_convert do |value|
				"Uconv.u8toeuc(#{value})"
			end
		end
		
		def def_from_euc_jp_to_utf_8
      begin
        require "uconv"
      rescue LoadError
        raise UnknownConvertMethod.new('euc_jp', 'utf-8')
      end
			def_convert do |value|
				"Uconv.euctou8(#{value})"
			end
		end
		
		def def_from_utf_8_to_shift_jis
      begin
        require "uconv"
      rescue LoadError
        raise UnknownConvertMethod.new('utf_8', 'shift_jis')
      end
			def_convert do |value|
				"Uconv.u8tosjis(#{value})"
			end
		end
		
		def def_from_shift_jis_to_utf_8
      begin
        require "uconv"
      rescue LoadError
        raise UnknownConvertMethod.new('shift_jis', 'utf_8')
      end
			def_convert do |value|
				"Uconv.sjistou8(#{value})"
			end
		end
		
		def def_from_shift_jis_to_euc_jp
      begin
        require "nkf"
      rescue LoadError
        raise UnknownConvertMethod.new('shift_jis', 'euc_jp')
      end
			def_convert do |value|
				"NKF.nkf('-Se', #{value})"
			end
		end
		
		def def_from_euc_jp_to_shift_jis
      begin
        require "nkf"
      rescue LoadError
        raise UnknownConvertMethod.new('euc_jp', 'shift_jis')
      end
			require "nkf"
			def_convert do |value|
				"NKF.nkf('-Es', #{value})"
			end
		end
		
		def def_from_iso_2022_jp_to_euc_jp
      begin
        require "nkf"
      rescue LoadError
        raise UnknownConvertMethod.new('iso_2022_jp', 'euc_jp')
      end
			require "nkf"
			def_convert do |value|
				"NKF.nkf('-Je', #{value})"
			end
		end
		
		def def_from_euc_jp_to_iso_2022_jp
      begin
        require "nkf"
      rescue LoadError
        raise UnknownConvertMethod.new('euc_jp', 'iso_2022_jp')
      end
			def_convert do |value|
				"NKF.nkf('-Ej', #{value})"
			end
		end

		def def_from_iso_8859_1_to_utf_8
			def_convert do |value|
				"#{value}.unpack('C*').pack('U*')"
			end
		end
		
		def def_from_utf_8_to_iso_8859_1
			def_convert do |value|
				<<-EOC
				array_utf8 = #{value}.unpack('U*')
				array_enc = []
				array_utf8.each do |num|
					if num <= 0xFF
						array_enc << num
					else
						array_enc.concat "&\#\#{num};".unpack('C*')
					end
				end
				array_enc.pack('C*')
EOC
			end
		end
		
	end
	
end
