require "rss/1.0"

module RSS

	DC_PREFIX = 'dc'
	DC_URI = "http://purl.org/dc/elements/1.1/"
	
	RDF.install_ns(DC_PREFIX, DC_URI)

	module DublincoreModel

		extend BaseModel

		ELEMENTS = []

		%w(title description creator subject publisher
		   contributor date type format identifier source
		   language relation coverage rights).each do |x|
				install_text_element("#{DC_PREFIX}_#{x}")
		end

		def dc_validate(tags)
			counter = {}
			ELEMENTS.each do |x|
				counter[x] = 0
			end

			tags.each do |tag|
				key = "#{DC_PREFIX}_#{tag}"
				raise UnknownTagError.new(tag, DC_URI)  unless counter.has_key?(key)
				counter[key] += 1
				raise TooMuchTagError.new(tag, tag_name) if counter[key] > 1
			end
		end

		alias_method(:_dc_date=, :dc_date=)
		def dc_date=(new_value)
			unless new_value.kind_of?(Time)
				new_value = validate_dc_date(new_value)
			end

			class << new_value
				alias_method(:_to_s, :to_s) unless respond_to?(:_to_s)
				alias_method(:to_s, :iso8601)
			end

			self._dc_date = new_value
		end
		
		private
		def validate_dc_date(value)
			begin
				Time.iso8601(value)
			rescue ArgumentError
				raise NotAvailableValueError.new("date", value)
			end
		end

	end

	class RDF
		class Channel; include DublincoreModel; end
		class Image; include DublincoreModel; end
		class Item; include DublincoreModel; end
		class Textinput; include DublincoreModel; end
	end

	if const_defined? :BaseListener
		prefix_size = DC_PREFIX.size + 1
		DublincoreModel::ELEMENTS.each do |x|
			BaseListener.install_get_text_element(x[prefix_size..-1], DC_URI, "#{x}=")
		end
	end

end
