require "rwiki/rss-maneger"
require "rwiki/rss-page"

module RWiki
	module RSS

		module Topic

			extend ERB::Util

			class << self

				def clear
					@@maneger = ::RWiki::RSS::Maneger.new
					@@topics = {}
					@@number = DISPLAY_NUMBER
					@@characters = DISPLAY_CHARACTERS
					@@use_thread = false
					@@display = false
				end

				def forget(expire)
					::RWiki::RSS::Maneger.forget(expire)
				end

				def use_thread() @@use_thread end
				def use_thread=(new_value) @@use_thread = new_value end
				def number() @@number end
				def number=(new_value) @@number = new_value end
				def characters() @@characters end
				def characters=(new_value) @@characters = new_value end
				def display() @@display end
				def display=(new_value) @@display = new_value end

				def add_topic(uri, charset, name, expire)
					@@topics[uri] = [charset, name, expire]
				end
				
				def topics
					return '' unless @@display
						
					if @@use_thread
						arg = @@topics.collect {|uri, values| [uri, *values]}
						@@maneger.parallel_parse(arg)
					else
						@@topics.collect do |uri, values|
							@@maneger.parse(uri, *values)
						end
					end
						
					rv = ''
					@@maneger.each do |uri, channel, items, name|
						if @@topics.has_key?(uri) and !items.empty?
							rv << %Q!<div class="rss_topic">\n! <<
								%Q! <span class="rss_topic_title">! <<
								%Q!#{tta(channel, name)}</span>\n! <<
								%Q! <ul class="rss_topic_item">\n!
							items[0...@@number].each do |item|
								rv << %Q! <li class="rss_topic_item">! <<
									%Q!#{ttia(item, @@characters)}</li>\n!
							end
							rv << %Q! </ul>\n</div>\n!
						end
					end
					rv
				end

				private
				def make_topic_title_anchor(channel, name)
					name = channel.title if name.to_s =~ /\A\s*\z/
					%Q!<a href="#{h channel.link.to_s.strip}">#{h name}</a>!
				end
				alias tta make_topic_title_anchor
				
				def make_topic_item_anchor(item, characters)
					href = h(item.link.to_s.strip)
					rv = %Q!<a href="#{href}">#{h item.title.to_s}</a>!
					desc = item.description
					if desc
						rv << %Q! : #{desc[0...characters]} <a href="#{href}">more</a>!
					end
					rv
				end
				alias ttia make_topic_item_anchor
			end

			clear

			class Section < RWiki::Section
				def initialize(config, pattern)
					super(config, pattern)
					add_prop_loader(:rss, PropLoader.new)
					add_default_src_proc(method(:default_src))
				end

				RWiki::ERbLoader.new('default_src(name)', 'rss-topic.erd').load(self)
			end


			class Page < PageFormat
				private
				@rhtml = { :view => ERbLoader.new('view(pg)', 'rss-topic.rhtml') }
				reload_rhtml
			end

		end

	end
end
